@@ -26,6 +26,7 @@ install("textblob")
 install("fastdtw")
 ##----------------------------------------------IMPORTS---------------------------------------------------------------------------#	
 import json
+import itertools
 import ast
 import copy
 from fastdtw import fastdtw
@@ -746,15 +747,15 @@ def text_tweets_features_extraction(data_tweets):
 			if((tweets_date[j-1].date() == tweets_date[j].date()) & (j!=(len(text_tweets_user)-1))):                            
 				
 				#Create a list with the tweets of a day
-				text_tweets_day.append({"text": text_tweets_user[j-1],"lang": lang_text_tweet[j-1],"RT": is_RT_list[j-1]})
-				text_tweets_day.append({"text": text_tweets_user[j],"lang": lang_text_tweet[j], "RT": is_RT_list[j]})              
+				text_tweets_day.append({"text": " ".join(filter(lambda x:x[0:4]!='http', text_tweets_user[j-1].split())),"lang": lang_text_tweet[j-1],"RT": is_RT_list[j-1]})
+				text_tweets_day.append({"text": " ".join(filter(lambda x:x[0:4]!='http', text_tweets_user[j].split())),"lang": lang_text_tweet[j], "RT": is_RT_list[j]})              
 				tweet_date_day.append(tweets_date[j-1])
 				tweet_date_day.append(tweets_date[j])                         
 			else:
 				if(j==(len(text_tweets_user)-1)):                  
 					if(tweets_date[j-1].date() == tweets_date[j].date()):                     
-						text_tweets_day.append({"text": text_tweets_user[j-1],"lang": lang_text_tweet[j-1],"RT": is_RT_list[j-1]})
-						text_tweets_day.append({"text": text_tweets_user[j],"lang": lang_text_tweet[j], "RT": is_RT_list[j]})              
+						text_tweets_day.append({"text": " ".join(filter(lambda x:x[0:4]!='http', text_tweets_user[j-1].split())),"lang": lang_text_tweet[j-1],"RT": is_RT_list[j-1]})
+						text_tweets_day.append({"text": " ".join(filter(lambda x:x[0:4]!='http', text_tweets_user[j].split())),"lang": lang_text_tweet[j], "RT": is_RT_list[j]})           
 						tweet_date_day.append(tweets_date[j-1])
 						tweet_date_day.append(tweets_date[j])
 						                     
@@ -765,7 +766,7 @@ def text_tweets_features_extraction(data_tweets):
 						text_tweets_day = []
 						tweet_date_day = []
 					else:                      
-						text_tweets_day.append({"text": text_tweets_user[j-1],"lang": lang_text_tweet[j-1],"RT": is_RT_list[j-1]})                                   
+						text_tweets_day.append({"text": " ".join(filter(lambda x:x[0:4]!='http', text_tweets_user[j-1].split())),"lang": lang_text_tweet[j-1],"RT": is_RT_list[j-1]})				                                   
 						tweet_date_day.append(tweets_date[j-1]) 
 						num_tw_days.append(1)                     
 						text_tweets_days.append([i for n, i in enumerate(text_tweets_day) if i not in text_tweets_day[n + 1:]])
@@ -773,7 +774,7 @@ def text_tweets_features_extraction(data_tweets):
 						text_tweets_day = []
 						tweet_date_day = []                      
 						tweet_date_day.append(tweets_date[j])                     
-						text_tweets_day.append({"text": text_tweets_user[j],"lang": lang_text_tweet[j], "RT": is_RT_list[j]}) 
+						text_tweets_day.append({"text": " ".join(filter(lambda x:x[0:4]!='http', text_tweets_user[j].split())),"lang": lang_text_tweet[j], "RT": is_RT_list[j]})              
 						num_tw_days.append(1)
 						text_tweets_days.append([i for n, i in enumerate(text_tweets_day) if i not in text_tweets_day[n + 1:]] )
 						tweet_date_days.append(list(dict.fromkeys(tweet_date_day)))
@@ -782,7 +783,7 @@ def text_tweets_features_extraction(data_tweets):
 						tweet_date_day = []                      
 				else:
 					if(num_tw_day ==1):
-						text_tweets_day.append({"text": text_tweets_user[j-1],"lang": lang_text_tweet[j-1],"RT": is_RT_list[j-1]})
+						text_tweets_day.append({"text": " ".join(filter(lambda x:x[0:4]!='http', text_tweets_user[j-1].split())),"lang": lang_text_tweet[j-1],"RT": is_RT_list[j-1]})	
 						tweet_date_day.append(tweets_date[j-1])                  
 					text_tweets_days.append([i for n, i in enumerate(text_tweets_day) if i not in text_tweets_day[n + 1:]] )
 					tweet_date_days.append(list(dict.fromkeys(tweet_date_day)))
@@ -870,6 +871,38 @@ def time_series_tw (df):
     return df 
 
 #-------------------------------------------Ratios and other advanced features----------------------------------#
+def duplicates_detection(data, i):
+    num_duplicates_tweets = []
+    if i < len(data):
+        possible_duplicated = []
+        for j in range(len(data)):
+            if j !=i:
+                list_dtw_1 = np.array(data["tw_per_day_list"].iloc[i])              
+                list_dtw_2 = np.array(data["tw_per_day_list"].iloc[j])
+                distance, path = fastdtw(list_dtw_1,list_dtw_2)
+                if (distance < 600):
+                    possible_duplicated.append({"screen_name":data["screen_name"].iloc[j], "distance": distance})
+        if (len(possible_duplicated)==0):
+            possible_duplicated.append({"screen_name": "Empty", "distance": "Empty"})           
+        data["possible_duplicates"].iloc[i] = possible_duplicated
+        if (data["possible_duplicates"].iloc[i][0]["screen_name"] =="Empty"):
+            num_duplicates_tweets_list.append(0)
+        else:          
+            for t in range(len(data["possible_duplicates"].iloc[i])):
+                text_1_aux = pd.DataFrame({"text":data[data["screen_name"] == data["possible_duplicates"].iloc[i][t]["screen_name"]]["text_tweets_days_list"]})
+                text_1 = pd.DataFrame()
+                for a in range(len(text_1_aux)):
+                    aux2 = pd.DataFrame({'text':text_1_aux["text"].iloc[a]})
+                    text_1 = text_1.append(aux2, ignore_index=False).reset_index(drop=True)
+                text_1 = list(itertools.chain(*text_1["text"]))
+                text_2 = list(itertools.chain(*data.iloc[i]["text_tweets_days_list"]))
+                comparacion = [item for item in text_1 if item in text_2]
+                if len(comparacion) > 0:
+                    num_duplicates_tweets.append(len(comparacion))
+                else:
+                    num_duplicates_tweets.append(0)
+    return num_duplicates_tweets, data
+	
 def advanced_features(data):
 
     #Posible bot
@@ -880,8 +913,11 @@ def advanced_features(data):
     data["follow_rate"] = 0
     data["max_tw_day"] = 0
     data["index_max_day_tw"]=0
+    data["num_duplicates_list"] = 0
     data["possible_duplicates"] = 0
+    num_duplicates_tweets_list = []
     for i in range(len(data)):
+        print(i)
         if (len(str(data.iloc[i]["screen_name"])) > 4):
             if((data.iloc[i]["screen_name"][-1].isdigit()) & (data.iloc[i]["screen_name"][-2].isdigit()) & 
                 (data.iloc[i]["screen_name"][-3].isdigit()) &(data.iloc[i]["screen_name"][-4].isdigit())):
@@ -906,21 +942,12 @@ def advanced_features(data):
         else:
             data["max_ret_tw"].iloc[i] = max((data["retweet_tweets_list"].iloc[i]))
           
-        if i < len(data):
-          possible_duplicated = []
-          for j in range(len(data)):
-            if j !=i:
-              list_dtw_1 = np.array(data["tw_per_day_list"].iloc[i])              
-              list_dtw_2 = np.array(data["tw_per_day_list"].iloc[j])
-              distance, path = fastdtw(list_dtw_1/max(max(list_dtw_1),max(list_dtw_2)),list_dtw_2/max(max(list_dtw_1),max(list_dtw_2)))
-              if (distance < 1.5):
-                possible_duplicated.append({"screen_name":data["screen_name"].iloc[j], "distance": distance})
-        if len(possible_duplicated)==0:
-          possible_duplicated.append({"screen_name": "Empty", "distance": "Empty"})           
-        data["possible_duplicates"].iloc[i] = possible_duplicated
+        num_duplicates_tweets, data = duplicates_detection(data, i)
+        num_duplicates_tweets_list.append(num_duplicates_tweets)
         data["max_tw_day"].iloc[i] =max((data["tw_per_day_list"].iloc[i]))
         data["days_with_tw"].iloc[i] = len(data["tw_per_day_list"].iloc[i])
         data["index_max_day_tw"].iloc[i] = data["tw_per_day_list"].iloc[i].index(data["max_tw_day"].iloc[i])
+    data["num_duplicates_list"]= num_duplicates_tweets_list
     data["follow_rate"][data["followers"]== 0] = np.median(data["follow_rate"])
     data["num_userTw"]= data["num_ownTw"] + data["RT"]
     data["tw_RT_rate"] = ((((data["num_ownTw"]))) / (data["num_userTw"])).replace([np.inf, -np.inf], np.nan).fillna(0)
@@ -1207,4 +1234,6 @@ def separate_text_tweets_feature(df_metadata):
     df_metadata["text_tweets_days_list"].iloc[k] = text_tweets_days    
     df_metadata["lang_days_list"].iloc[k] = lang_text_tweets_days
     df_metadata["RT_days_list"].iloc[k]= RT_text_tweets_days
-  return df_metadata
\ No newline at end of file
+  return df_metadata
+  
+  
\ No newline at end of file
